@@ -597,6 +597,8 @@ void pl_color_space_nominal_luma_ex(const struct pl_nominal_luma_params *params)
         avg_luma = pl_hdr_rescale(PL_HDR_PQ, scaling, csp->hdr.avg_pq_y);
     }
 
+    if (min_luma < 0)
+        min_luma = 0; // sanity
     if (max_luma && min_luma > max_luma) // sanity
         min_luma = max_luma = 0;
 
@@ -644,8 +646,9 @@ void pl_color_space_infer(struct pl_color_space *space)
         .color      = space,
         .metadata   = PL_HDR_METADATA_HDR10,
         .scaling    = PL_HDR_NITS,
-        .out_min    = &space->hdr.min_luma,
         .out_max    = &space->hdr.max_luma,
+        // Preserve tagged minimum
+        .out_min    = space->hdr.min_luma ? NULL : &space->hdr.min_luma,
     ));
 
     // Default the signal color space based on the nominal raw primaries
