@@ -72,6 +72,7 @@ void pl_shader_reset(pl_shader sh, const struct pl_shader_params *params)
 
         // Preserve array allocations
         .tmp.elem       = sh->tmp.elem,
+        .obj.elem       = sh->obj.elem,
         .vas.elem       = sh->vas.elem,
         .vars.elem      = sh->vars.elem,
         .descs.elem     = sh->descs.elem,
@@ -247,8 +248,6 @@ ident_t sh_desc(pl_shader sh, struct pl_shader_desc sd)
     switch (sd.desc.type) {
     case PL_DESC_BUF_UNIFORM:
     case PL_DESC_BUF_STORAGE:
-    case PL_DESC_BUF_TEXEL_UNIFORM:
-    case PL_DESC_BUF_TEXEL_STORAGE:
         // Skip re-attaching the same buffer desc twice
         // FIXME: define aliases if the variable names differ
         for (int i = 0; i < sh->descs.num; i++) {
@@ -263,6 +262,8 @@ ident_t sh_desc(pl_shader sh, struct pl_shader_desc sd)
         sd.buffer_vars = pl_memdup(SH_TMP(sh), sd.buffer_vars, bsize);
         break;
 
+    case PL_DESC_BUF_TEXEL_UNIFORM:
+    case PL_DESC_BUF_TEXEL_STORAGE:
     case PL_DESC_SAMPLED_TEX:
     case PL_DESC_STORAGE_IMG:
         pl_assert(!sd.num_buffer_vars);
@@ -579,6 +580,11 @@ ident_t sh_subpass(pl_shader sh, const pl_shader sub)
     pl_str_builder_concat(sh->buffers[SH_BUF_HEADER], sub->buffers[SH_BUF_BODY]);
     GLSLH("%s\n}\n\n", retvals[sub->res.output]);
 
+    // Ref all objects
+    PL_ARRAY_CONCAT(sh, sh->obj, sub->obj);
+    for (int i = 0; i < sub->obj.num; i++)
+        pl_rc_ref(&sub->obj.elem[i]->rc);
+
     // Copy over all of the descriptors etc.
     for (int i = 0; i < sub->tmp.num; i++)
         PL_ARRAY_APPEND(sh, sh->tmp, pl_ref_dup(sub->tmp.elem[i]));
