@@ -20,6 +20,8 @@
 
 #include "shaders.h"
 
+#include <libplacebo/shaders/lut.h>
+
 static inline bool isnumeric(char c)
 {
     return (c >= '0' && c <= '9') || c == '-';
@@ -458,7 +460,9 @@ next_dim: ; // `continue` out of the inner loop
     update |= method != lut->method;
 
     if (update) {
-        PL_DEBUG(sh, "LUT cache invalidated, regenerating..");
+        PL_MSG(sh, params->dynamic ? PL_LOG_TRACE : PL_LOG_DEBUG,
+               "LUT cache invalidated, regenerating..");
+
         size_t buf_size = size * params->comps * pl_var_type_size(vartype);
         tmp = pl_zalloc(NULL, buf_size);
         params->fill(tmp, params);
@@ -655,7 +659,7 @@ next_dim: ; // `continue` out of the inner loop
         GLSLH("const %s %s[%d] = %s[](\n  ",
               vartypes[vartype][params->comps - 1], arr_name, size,
               vartypes[vartype][params->comps - 1]);
-        pl_str_append(sh, &sh->buffers[SH_BUF_HEADER], lut->str);
+        sh_append_str(sh, SH_BUF_HEADER, lut->str);
         GLSLH(");\n");
         break;
 
