@@ -2,6 +2,8 @@
 #include "d3d11/gpu.h"
 #include <dxgi1_2.h>
 
+#include <libplacebo/d3d11.h>
+
 int main()
 {
     pl_log log = pl_test_logger();
@@ -18,7 +20,11 @@ int main()
     if (!pCreateDXGIFactory1)
         return SKIP;
 
-    pCreateDXGIFactory1(&IID_IDXGIFactory1, (void **) &factory);
+    hr = pCreateDXGIFactory1(&IID_IDXGIFactory1, (void **) &factory);
+    if (FAILED(hr)) {
+        printf("Failed to create DXGI factory\n");
+        return SKIP;
+    }
 
     // Test all attached devices
     for (int i = 0;; i++) {
@@ -38,7 +44,7 @@ int main()
         }
         SAFE_RELEASE(adapter1);
 
-        const struct pl_d3d11 *d3d11 = pl_d3d11_create(log, pl_d3d11_params(
+        const struct pl_d3d11_t *d3d11 = pl_d3d11_create(log, pl_d3d11_params(
             .debug = true,
             .adapter_luid = desc.AdapterLuid,
         ));
