@@ -74,15 +74,7 @@ static void pass_destroy_cb(pl_gpu gpu, pl_pass pass)
 
 void vk_pass_destroy(pl_gpu gpu, pl_pass pass)
 {
-    struct pl_vk *p = PL_PRIV(gpu);
-    struct vk_ctx *vk = p->vk;
-    pl_mutex_lock(&p->recording);
-    if (p->cmd) {
-        vk_cmd_callback(p->cmd, (vk_cb) pass_destroy_cb, gpu, pass);
-    } else {
-        vk_dev_callback(vk, (vk_cb) pass_destroy_cb, gpu, pass);
-    }
-    pl_mutex_unlock(&p->recording);
+    vk_gpu_idle_callback(gpu, (vk_cb) pass_destroy_cb, gpu, pass);
 }
 
 static const VkDescriptorType dsType[] = {
@@ -191,6 +183,8 @@ static VkResult vk_recreate_pipelines(struct vk_ctx *vk, pl_pass pass,
     // The old pipeline might still be in use, so we have to destroy it
     // asynchronously with a device idle callback
     if (*out_pipe) {
+        // We don't need to use `vk_gpu_idle_callback` because the only command
+        // that can access a VkPipeline, `vk_pass_run`, always flushes `p->cmd`.
         vk_dev_callback(vk, (vk_cb) destroy_pipeline, vk, vk_wrap_handle(*out_pipe));
         *out_pipe = VK_NULL_HANDLE;
     }
@@ -367,7 +361,7 @@ pl_pass vk_pass_create(pl_gpu gpu, const struct pl_pass_params *params)
 
 #define NUM_DS (PL_ARRAY_SIZE(pass_vk->dss))
 
-    static int dsSize[PL_DESC_TYPE_COUNT] = {0};
+    int dsSize[PL_DESC_TYPE_COUNT] = {0};
     VkDescriptorSetLayoutBinding *bindings = pl_calloc_ptr(tmp, num_desc, bindings);
 
     uint32_t max_tex = vk->limits.maxPerStageDescriptorSampledImages,
@@ -706,8 +700,9 @@ static void vk_update_descriptor(pl_gpu gpu, struct vk_cmd *cmd, pl_pass pass,
         struct pl_tex_vk *tex_vk = PL_PRIV(tex);
 
         vk_tex_barrier(gpu, cmd, tex, passStages[pass->params.type],
-                      VK_ACCESS_SHADER_READ_BIT,
-                      VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL, false);
+                       VK_ACCESS_SHADER_READ_BIT,
+                       VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL,
+                       VK_QUEUE_FAMILY_IGNORED);
 
         VkDescriptorImageInfo *iinfo = &pass_vk->dsiinfo[idx];
         *iinfo = (VkDescriptorImageInfo) {
@@ -724,7 +719,8 @@ static void vk_update_descriptor(pl_gpu gpu, struct vk_cmd *cmd, pl_pass pass,
         struct pl_tex_vk *tex_vk = PL_PRIV(tex);
 
         vk_tex_barrier(gpu, cmd, tex, passStages[pass->params.type],
-                       access[desc->access], VK_IMAGE_LAYOUT_GENERAL, false);
+                       access[desc->access], VK_IMAGE_LAYOUT_GENERAL,
+                       VK_QUEUE_FAMILY_IGNORED);
 
         VkDescriptorImageInfo *iinfo = &pass_vk->dsiinfo[idx];
         *iinfo = (VkDescriptorImageInfo) {
@@ -952,9 +948,14 @@ void vk_pass_run(pl_gpu gpu, const struct pl_pass_run_params *params)
                                    index_fmts[params->index_fmt]);
         }
 
+
+        VkAccessFlags fbo_access = VK_ACCESS_COLOR_ATTACHMENT_WRITE_BIT;
+        if (pass->params.load_target)
+            fbo_access |= VK_ACCESS_COLOR_ATTACHMENT_READ_BIT;
+
         vk_tex_barrier(gpu, cmd, tex, VK_PIPELINE_STAGE_COLOR_ATTACHMENT_OUTPUT_BIT,
-                       VK_ACCESS_COLOR_ATTACHMENT_WRITE_BIT,
-                       VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL, false);
+                       fbo_access, VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL,
+                       VK_QUEUE_FAMILY_IGNORED);
 
         VkViewport viewport = {
             .x = params->viewport.x0,
