@@ -29,6 +29,8 @@ struct sem_pair {
 };
 
 struct priv {
+    struct pl_sw_fns impl;
+
     pl_mutex lock;
     struct vk_ctx *vk;
     VkSurfaceKHR surf;
@@ -54,7 +56,7 @@ struct priv {
     int last_imgidx;                // the image index last acquired (for submit)
 };
 
-static struct pl_sw_fns vulkan_swapchain;
+static const struct pl_sw_fns vulkan_swapchain;
 
 static bool map_color_space(VkColorSpaceKHR space, struct pl_color_space *out)
 {
@@ -271,12 +273,21 @@ static void set_hdr_metadata(struct priv *p, const struct pl_hdr_metadata *metad
     if (!vk->SetHdrMetadataEXT)
         return;
 
+    // Whitelist only values that we support signalling metadata for
+    struct pl_hdr_metadata fix = {
+        .prim     = metadata->prim,
+        .min_luma = metadata->min_luma,
+        .max_luma = metadata->max_luma,
+        .max_cll  = metadata->max_cll,
+        .max_fall = metadata->max_fall,
+    };
+
     // Ignore no-op changes
-    if (pl_hdr_metadata_equal(metadata, &p->hdr_metadata))
+    if (pl_hdr_metadata_equal(&fix, &p->hdr_metadata))
         return;
 
     // Remember the metadata so we can re-apply it after swapchain recreation
-    p->hdr_metadata = *metadata;
+    p->hdr_metadata = fix;
 
     // Ignore HDR metadata requests for SDR swapchains
     if (!pl_color_transfer_is_hdr(p->color_space.transfer))
@@ -287,18 +298,18 @@ static void set_hdr_metadata(struct priv *p, const struct pl_hdr_metadata *metad
 
     vk->SetHdrMetadataEXT(vk->dev, 1, &p->swapchain, &(VkHdrMetadataEXT) {
         .sType = VK_STRUCTURE_TYPE_HDR_METADATA_EXT,
-        .displayPrimaryRed   = { metadata->prim.red.x,   metadata->prim.red.y },
-        .displayPrimaryGreen = { metadata->prim.green.x, metadata->prim.green.y },
-        .displayPrimaryBlue  = { metadata->prim.blue.x,  metadata->prim.blue.y },
-        .whitePoint = { metadata->prim.white.x, metadata->prim.white.y },
-        .maxLuminance = metadata->max_luma,
-        .minLuminance = metadata->min_luma,
-        .maxContentLightLevel = metadata->max_cll,
-        .maxFrameAverageLightLevel = metadata->max_fall,
+        .displayPrimaryRed   = { fix.prim.red.x,   fix.prim.red.y },
+        .displayPrimaryGreen = { fix.prim.green.x, fix.prim.green.y },
+        .displayPrimaryBlue  = { fix.prim.blue.x,  fix.prim.blue.y },
+        .whitePoint = { fix.prim.white.x, fix.prim.white.y },
+        .maxLuminance = fix.max_luma,
+        .minLuminance = fix.min_luma,
+        .maxContentLightLevel = fix.max_cll,
+        .maxFrameAverageLightLevel = fix.max_fall,
     });
 
     // Keep track of applied HDR colorimetry metadata
-    p->color_space.hdr = *metadata;
+    p->color_space.hdr = p->hdr_metadata;
 }
 
 pl_swapchain pl_vulkan_create_swapchain(pl_vulkan plvk,
@@ -313,12 +324,12 @@ pl_swapchain pl_vulkan_create_swapchain(pl_vulkan plvk,
     }
 
     struct pl_swapchain_t *sw = pl_zalloc_obj(NULL, sw, struct priv);
-    sw->impl = &vulkan_swapchain;
     sw->log = vk->log;
     sw->gpu = gpu;
 
     struct priv *p = PL_PRIV(sw);
     pl_mutex_init(&p->lock);
+    p->impl = vulkan_swapchain;
     p->params = *params;
     p->vk = vk;
     p->surf = params->surface;
@@ -566,10 +577,10 @@ static bool vk_sw_recreate(pl_swapchain sw, int w, int h)
     VkImage *vkimages = NULL;
     uint32_t num_images = 0;
 
-    VkSwapchainCreateInfoKHR sinfo = p->protoInfo;
-    if (!update_swapchain_info(p, &sinfo, w, h))
+    if (!update_swapchain_info(p, &p->protoInfo, w, h))
         return false;
 
+    VkSwapchainCreateInfoKHR sinfo = p->protoInfo;
 #ifdef VK_EXT_full_screen_exclusive
     // Explicitly disallow full screen exclusive mode if possible
     static const VkSurfaceFullScreenExclusiveInfoEXT fsinfo = {
@@ -706,9 +717,12 @@ static bool vk_sw_start_frame(pl_swapchain sw,
             // fall through
         case VK_SUCCESS:
             p->last_imgidx = imgidx;
-            pl_vulkan_release(sw->gpu, p->images.elem[imgidx],
-                              VK_IMAGE_LAYOUT_UNDEFINED,
-                              (pl_vulkan_sem){ sem_in });
+            pl_vulkan_release_ex(sw->gpu, pl_vulkan_release_params(
+                .tex        = p->images.elem[imgidx],
+                .layout     = VK_IMAGE_LAYOUT_UNDEFINED,
+                .qf         = VK_QUEUE_FAMILY_IGNORED,
+                .semaphore  = { sem_in },
+            ));
             *out_frame = (struct pl_swapchain_frame) {
                 .fbo = p->images.elem[imgidx],
                 .flipped = false,
@@ -757,9 +771,13 @@ static bool vk_sw_submit_frame(pl_swapchain sw)
     p->idx_sems %= p->sems.num;
     p->last_imgidx = -1;
 
-    bool held = pl_vulkan_hold(gpu, p->images.elem[idx],
-                               VK_IMAGE_LAYOUT_PRESENT_SRC_KHR,
-                               (pl_vulkan_sem){ sem_out });
+    bool held = pl_vulkan_hold_ex(gpu, pl_vulkan_hold_params(
+        .tex        = p->images.elem[idx],
+        .layout     = VK_IMAGE_LAYOUT_PRESENT_SRC_KHR,
+        .qf         = VK_QUEUE_FAMILY_IGNORED,
+        .semaphore  = { sem_out },
+    ));
+
     if (!held) {
         PL_ERR(gpu, "Failed holding swapchain image for presentation");
         pl_mutex_unlock(&p->lock);
@@ -872,7 +890,7 @@ bool pl_vulkan_swapchain_suboptimal(pl_swapchain sw)
     return p->suboptimal;
 }
 
-static struct pl_sw_fns vulkan_swapchain = {
+static const struct pl_sw_fns vulkan_swapchain = {
     .destroy            = vk_sw_destroy,
     .latency            = vk_sw_latency,
     .resize             = vk_sw_resize,
