@@ -34,6 +34,9 @@ vk_mem_handle_type(enum pl_handle_type handle_type)
         return VK_EXTERNAL_MEMORY_HANDLE_TYPE_DMA_BUF_BIT_EXT;
     case PL_HANDLE_HOST_PTR:
         return VK_EXTERNAL_MEMORY_HANDLE_TYPE_HOST_ALLOCATION_BIT_EXT;
+    case PL_HANDLE_MTL_TEX:
+    case PL_HANDLE_IOSURFACE:
+        return 0;
     }
 
     pl_unreachable();
@@ -54,6 +57,8 @@ vk_sync_handle_type(enum pl_handle_type handle_type)
         return VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_OPAQUE_WIN32_KMT_BIT_KHR;
     case PL_HANDLE_DMA_BUF:
     case PL_HANDLE_HOST_PTR:
+    case PL_HANDLE_MTL_TEX:
+    case PL_HANDLE_IOSURFACE:
         return 0;
     }
 
@@ -122,7 +127,7 @@ const void *vk_find_struct(const void *chain, VkStructureType stype)
     return NULL;
 }
 
-void vk_link_struct(void *chain, void *in)
+void vk_link_struct(void *chain, const void *in)
 {
     if (!in)
         return;
@@ -131,7 +136,7 @@ void vk_link_struct(void *chain, void *in)
     while (out->pNext)
         out = out->pNext;
 
-    out->pNext = in;
+    out->pNext = (void *) in;
 }
 
 void *vk_struct_memdup(void *alloc, const void *pin)
@@ -141,8 +146,7 @@ void *vk_struct_memdup(void *alloc, const void *pin)
 
     const VkBaseInStructure *in = pin;
     size_t size = vk_struct_size(in->sType);
-    if (!size)
-        return NULL;
+    pl_assert(size);
 
     VkBaseOutStructure *out = pl_memdup(alloc, in, size);
     out->pNext = NULL;
@@ -151,10 +155,12 @@ void *vk_struct_memdup(void *alloc, const void *pin)
 
 void *vk_chain_memdup(void *alloc, const void *pin)
 {
+    if (!pin)
+        return NULL;
+
     const VkBaseInStructure *in = pin;
     VkBaseOutStructure *out = vk_struct_memdup(alloc, in);
-    if (!out)
-        return NULL;
+    pl_assert(out);
 
     out->pNext = vk_chain_memdup(alloc, in->pNext);
     return out;
